#include <iostream>
#include <stdio.h>
#include "poker.h"
#include "arrays.h"

using namespace std;
int deck[52];

int findit( int key )
{
    int low = 0, high = 4887, mid;

    while ( low <= high )
    {
        mid = (high+low) >> 1;      // divide by two
        if ( key < products[mid] )
            high = mid - 1;
        else if ( key > products[mid] )
            low = mid + 1;
        else
            return( mid );
    }
    fprintf( stderr, "ERROR:  no match found; key = %d\n", key );
    return( -1 );
}
void init_deck( int *deck )
{
    int i, j, n = 0, suit = 0x8000;

    for ( i = 0; i < 4; i++, suit >>= 1 )
        for ( j = 0; j < 13; j++, n++ )
            deck[n] = primes[j] | (j << 8) | suit | (1 << (16+j));
}

int
find_card( int rank, int suit, int *deck )
{
	int i, c;

	for ( i = 0; i < 52; i++ )
	{
		c = deck[i];
		if ( (c & suit)  &&  (RANK(c) == rank) )
			return( i );
	}
	return( -1 );
}


void print_hand( int *hand, int n )
{
    int i, r;
    char suit;
    static char *rank = "23456789TJQKA";

    for ( i = 0; i < n; i++ )
    {
        r = (*hand >> 8) & 0xF;
        if ( *hand & 0x8000 )
            suit = 'c';
        else if ( *hand & 0x4000 )
            suit = 'd';
        else if ( *hand & 0x2000 )
            suit = 'h';
        else
            suit = 's';

        printf( "%c%c ", rank[r], suit );
        hand++;
    }
}


int
hand_rank( short val )
{
    if (val > 6185) return(HIGH_CARD);        // 1277 high card
    if (val > 3325) return(ONE_PAIR);         // 2860 one pair
    if (val > 2467) return(TWO_PAIR);         //  858 two pair
    if (val > 1609) return(THREE_OF_A_KIND);  //  858 three-kind
    if (val > 1599) return(STRAIGHT);         //   10 straights
    if (val > 322)  return(FLUSH);            // 1277 flushes
    if (val > 166)  return(FULL_HOUSE);       //  156 full house
    if (val > 10)   return(FOUR_OF_A_KIND);   //  156 four-kind
    return(STRAIGHT_FLUSH);                   //   10 straight-flushes
}


short
eval_5cards( int c1, int c2, int c3, int c4, int c5 )
{
    int q;
    short s;

    q = (c1|c2|c3|c4|c5) >> 16;

    /* check for Flushes and StraightFlushes
    */
    if ( c1 & c2 & c3 & c4 & c5 & 0xF000 )
	return( flushes[q] );

    /* check for Straights and HighCard hands
    */
    s = unique5[q];
    if ( s )  return ( s );

    /* let's do it the hard way
    */
    q = (c1&0xFF) * (c2&0xFF) * (c3&0xFF) * (c4&0xFF) * (c5&0xFF);
    q = findit( q );

    return( values[q] );
}


short
eval_5hand( int *hand )
{
    int c1, c2, c3, c4, c5;

    c1 = *hand++;
    c2 = *hand++;
    c3 = *hand++;
    c4 = *hand++;
    c5 = *hand;

    return( eval_5cards(c1,c2,c3,c4,c5) );
}


short
eval_7hand( int *hand )
{
    int i, j, q, best = 9999, subhand[5];

	for ( i = 0; i < 21; i++ )
	{
		for ( j = 0; j < 5; j++ )
			subhand[j] = hand[ perm7[i][j] ];
		q = eval_5hand( subhand );
		if ( q < best )
			best = q;
	}
	return( best );
}

int finddeckindex(char suit , char card)
{
    int s ,c;
    if (suit=='c'){s=0;}
    else if (suit=='d'){s=1;}
    else if (suit=='h'){s=2;}
    else {s=3;}

    if(card=='A'){c=12;}
    else if(card=='K'){c=11;}
    else if(card=='Q'){c=10;}
    else if(card=='J'){c=9;}
    else if(card=='T'){c=8;}
    else {c=(card-'0')-2  ;}

    return s*13+c;
}
float handStrength(int n1,int n2,int n3,int n4,int n5)
{
    int c1, c2, c3, c4, c5,m,n;
    int opprank,ourrank;
    float strength,a,b,c,e;
    //printf("\n ok till here 2 \n");


    c1 = deck[n1];
    c2 = deck[n1];
    c3 = deck[n3];
    c4 = deck[n4];
    c5 = deck[n5];

  ourrank=eval_5cards(c1,c2,c3,c4,c5);
  //printf("\n ok till here 3 \n");
  int ahead=0,tied=0,behind=0,handstrength=0;

for(int i=0;i<52;i++)
{
    for(int j=i+1;j<52;j++)
    {
        if(i==n1||i==n2||i==n3||i==n4||i==n5||j==n1||j==n2||j==n3||j==n4||j==n5)
        {
            continue;
        }
        else
        {
            m=deck[i];
            n=deck[j];
            opprank=eval_5cards(m,n,c3,c4,c5);
            if(opprank<ourrank)
            {
                ahead+=1;
            }
            else if(ourrank=opprank)
            {
                tied+=1;
            }
            else
            {
                behind+=1;
            }
        }

    }
}
       printf("ahead  is %d  \n",  ahead);
       printf("tied   is %d  \n",  tied);
       printf("behind is %d  \n",  behind);
       a=tied/2;
       b=ahead+a;
       c=ahead+tied+behind;
       e=b/c;

       strength=e;
       printf("strength is %f  \n",  strength);

       return strength;
}
float Ppot(int *hand)
{
    int HP[3][3];
    int HPTotal[3];
    int hand7[7];
    float p1,p2,p3,p4,p5,p6,p7,p8,p9;
    int m,n,u,v,ourrank,opprank,index,oppbest,ourbest,c1,c2,count;
    hand7[0]=deck[hand[0]];
    hand7[1]=deck[hand[1]];
    hand7[2]=deck[hand[2]];
    hand7[3]=deck[hand[3]];
    hand7[4]=deck[hand[4]];
    c1=hand7[0];
    c2=hand7[1];

    HPTotal[0]=0;
    HPTotal[1]=0;
    HPTotal[2]=0;
    for (int a=0;a<3;a++)
    {
        for(int b=0;b<3;b++)
        {
            HP[a][b]=0;
        }
    }
    ourrank=eval_5cards(hand7[0],hand7[1],hand7[2],hand7[3],hand7[4]);
    cout <<"ourrank " << ourrank <<endl;
    count=0;
    for(int i=0;i<52;i++)
    {
        if(i==hand[0]||i==hand[1]||i==hand[2]||i==hand[3]||i==hand[4] )
        {
            continue;
        }
        for(int j=i+1;j<52;j++)
        {
            if(j==hand[0]||j==hand[1]||j==hand[2]||j==hand[3]||j==hand[4])
            {
                continue;
            }
            else
            {
                count++;
                m=deck[i];
                n=deck[j];
                opprank=eval_5cards(m,n,hand7[2],hand7[3],hand7[4]);
                //cout<<"5 is ok\n";
                if(opprank<ourrank)
                {
                    index=0;
                }
               else if(ourrank==opprank)
                {
                    index=1;
                }
                else
                {
                    index=2;
                }
                HPTotal[index]+=1;

                for(int k=0;k<52;k++)
                {
                    if(k==hand[0]||k==hand[1]||k==hand[2]||k==hand[3]||k==hand[4]||k==i||k==j){continue;}
                    for(int l=k+1;l<52;l++)
                    {
                        if(l==hand[0]||l==hand[1]||l==hand[2]||l==hand[3]||l==hand[4]||l==i||l==j){continue;}
                        else
                        {
                        hand7[0]=c1;
                        hand7[1]=c2;
                        hand7[5]=deck[k];
                        hand7[6]=deck[l];
                        ourbest=eval_7hand(hand7);
                        //cout << " ourbest "<< ourbest << endl ;
                        hand7[0]=m;
                            hand7[1]=n;
                        oppbest=eval_7hand(hand7);
                        //cout <<"7 is ok \n";
                        //if(oppbest<ourbest)
                        //{cout <<" oppbest " << oppbest <<endl;}
                        if(oppbest<ourbest)
                        {
                        HP[index][0]+=1;
                        }
                        else if(ourbest==oppbest)
                        {
                         HP[index][1]+=1;
                        }
                         else
                        {
                        HP[index][2]+=1;
                        }
                     }
                }
            }
        }
    }
}
cout << count << "count \n";

        p1=float(HP[1][2])/2;
        p2=float(HP[0][1])/2;
        p3=float(HP[0][2]);
        p4=float(HP[0][1])+float(HP[0][0])+float(HP[0][2]);
        p5=float(HP[1][0])+float(HP[1][1])+float(HP[1][2]);
        if(p4>0)
        {
        p6=p2/p4;
        p7=p3/p4;
        }
        else
        {
            p6=0;
            p7=0;
        }
        if(p5>0)
        {
        p8=p1/p5;
        }
        else
        {
            p8=0;
        }
        p9=p6+p7+p8;
        cout << p1 << ":p1 " << p2 << ":p2 " << p3 << ":p3 " << p4 << ":p4 \n"<< p5 << ":p5 "<< p6 << ":p6 " << p7 << ":p7 " << p8 << ":p8 " << p9 << ":p9 " ;

            return p8;

}

float Npot(int *hand)
{
    int HP[3][3];
    int HPTotal[3];
    int hand7[7];
    float p1,p2,p3,p4,p5,p6,p7,p8,p9;
    int m,n,u,v,ourrank,opprank,index,oppbest,ourbest,c1,c2,count;
    hand7[0]=deck[hand[0]];
    hand7[1]=deck[hand[1]];
    hand7[2]=deck[hand[2]];
    hand7[3]=deck[hand[3]];
    hand7[4]=deck[hand[4]];
    c1=hand7[0];
    c2=hand7[1];

    HPTotal[0]=0;
    HPTotal[1]=0;
    HPTotal[2]=0;
    for (int a=0;a<3;a++)
    {
        for(int b=0;b<3;b++)
        {
            HP[a][b]=0;
        }
    }
    ourrank=eval_5cards(hand7[0],hand7[1],hand7[2],hand7[3],hand7[4]);
    cout <<"ourrank " << ourrank <<endl;
    count=0;
    for(int i=0;i<52;i++)
    {
        if(i==hand[0]||i==hand[1]||i==hand[2]||i==hand[3]||i==hand[4] )
        {
            continue;
        }
        for(int j=i+1;j<52;j++)
        {
            if(j==hand[0]||j==hand[1]||j==hand[2]||j==hand[3]||j==hand[4])
            {
                continue;
            }
            else
            {
                count++;
                m=deck[i];
                n=deck[j];
                opprank=eval_5cards(m,n,hand7[2],hand7[3],hand7[4]);
                //cout<<"5 is ok\n";
                if(opprank<ourrank)
                {
                    index=0;
                }
               else if(ourrank==opprank)
                {
                    index=1;
                }
                else
                {
                    index=2;
                }
                HPTotal[index]+=1;

                for(int k=0;k<52;k++)
                {
                    if(k==hand[0]||k==hand[1]||k==hand[2]||k==hand[3]||k==hand[4]||k==i||k==j){continue;}
                    for(int l=k+1;l<52;l++)
                    {
                        if(l==hand[0]||l==hand[1]||l==hand[2]||l==hand[3]||l==hand[4]||l==i||l==j){continue;}
                        else
                        {
                        hand7[0]=c1;
                        hand7[1]=c2;
                        hand7[5]=deck[k];
                        hand7[6]=deck[l];
                        ourbest=eval_7hand(hand7);
                        //cout << " ourbest "<< ourbest << endl ;
                        hand7[0]=m;
                            hand7[1]=n;
                        oppbest=eval_7hand(hand7);
                        //cout <<"7 is ok \n";
                        //if(oppbest<ourbest)
                        //{cout <<" oppbest " << oppbest <<endl;}
                        if(oppbest<ourbest)
                        {
                        HP[index][0]+=1;
                        }
                        else if(ourbest==oppbest)
                        {
                         HP[index][1]+=1;
                        }
                         else
                        {
                        HP[index][2]+=1;
                        }
                     }
                }
            }
        }
    }
}
cout << count << "count \n";

        p1=float(HP[2][1])/2;
        p2=float(HP[1][0])/2;
        p3=float(HP[2][0]);
        p4=float(HP[2][1])+float(HP[2][0])+float(HP[2][2]);
        p5=float(HP[1][0])+float(HP[1][1])+float(HP[1][2]);
        if(p4>0)
        {
        p6=p1/p4;
        p7=p3/p4;
        }
        else
        {
            p6=0;
            p7=0;
        }
        if(p5>0)
        {
        p8=p2/p5;
        }
        else
        {
            p8=0;
        }
        p9=p6+p7+p8;
        cout << p1 << ":p1 " << p2 << ":p2 " << p3 << ":p3 " << p4 << ":p4 \n"<< p5 << ":p5 "<< p6 << ":p6 " << p7 << ":p7 " << p8 << ":p8 " << p9 << ":p9 " ;

            return p8;

}
float EHS(int *hand)
{
    int c1,c2,c3,c4,c5;
    float S,P,N,E;
    c1=hand[0];
    c2=hand[1];
    c3=hand[2];
    c4=hand[3];
    c5=hand[4];
    S=handStrength(c1,c2,c3,c4,c5);
    P=Ppot(hand);
    N=Npot(hand);
    E=S*(1-N)+(1-S)*P;

    return E;

}

